      SUBROUTINE SELECT_UNIQUE(cjobid, iunit_t, nscan, jdate, atime,
     &                         istrt, istop, do_debug)
C***********************************************************************
C* SELECT_UNIQUE  Select unique Wentz NRT scans to process             *
C* SSMIS version follows original FNMOC NRT SSMI code, using jdate and *
C* atime.  (TRMM V6 SSMI code used "itime87".)                         *
C*								       *
C* This routine selects NRT scans to process and maintains the list    *
C* of times.  A "contact" is an orbit segment.  We assume that contacts*
C* are sufficiently disjoint that there are no multiple unique runs of *
C* scans separated by runs of overlap.  However, testing revealed that *
C* in rare instances where this does occur, the data dropouts were on  *
C* the order of a few seconds.                                         *
C*                                                                     *
C* Files are considered contiguous if there is less than a 1-minute    *
C* (MXGP = 60 seconds) between the end time of one and the start time  *
C* of the next.  A warning is generated if the contact is more than 1  *
C* month past the current last-processed contact entry.		       *
C*								       *
C* Contact time file format:					       *
C*   nct            						       *
C*   kct (1,1)   kct (2,1)					       *
C*    ...         ...						       *
C*   kct (1,nct) kct (2, nct)					       *
C* where							       *
C*   nct = number of entries in list				       *
C*   kct = start and stop times in seconds since 1/1/1987 of unique    *
C*         contact data					               *
C*								       *
C* Initial file values:						       *
C*   1                                                                 *
C*           0          0                                              *
C*                                                                     *
C* Input Parameters:                                                   *
C*                                                                     *
C*  cjobid      CHAR    PPS job ID
C*  iunit_t     INT     unit number of dynamically-built file          *
C*                      containing start/end times of recently-        *
C*                      processed NRT files                            *
C*  nscan       INT     total number of high-res scans in the NRT file *
C*  jdate       INT     array of Julian days for each scan (YYYYDDD)   *
C*  atime       REAL    array of seconds-into-day for each scan        *
C*  (NOT USED IN THIS VERSION:                                         *
C*   itime87     INT     array of seconds since Jan. 1, 1987 )         *
C*                                                                     *
C* Output Parameters:                                                  *
C*                                                                     *
C*  istrt       INT     record number of first unique scan             *
C*  istop       INT     record number of last unique scan              *
C*								       *
C**								       *
C* Log:								       *
C* G.Huffman/SSAI	11/01					       *
C* E.Nelkin/SSAI        11/01  Called by INGEST_SDR                    *
C* E.Nelkin/SSAI         7/02  Wentz NRT version; called by READNRT    *
C* E.Nelkin/SSAI         4/03  Compute length of timefile string       *
C* E.Nelkin/SSAI         3/04  Additional TSDIS error codes            *
C* G.Huffman/SSAI	 1/07  Fix indexing for pitching "old" entries,*
C*			       change MXCT 20->99, MXGAP 600->60; test *
C*			       for far-in-the-future contact	       *
C* E.Nelkin/SSAI         8/09  Close the time file at the end          *
C* E.Nelkin/SSAI         5/11  SSMIS version; use jdate, atime instead *
C*                             of itime87; dimension as MAX_SCANS      *
C* E.Nelkin/SSAI        10/11  Version compatible with GPROF2004V: pass*
C*                             in iunit_t instead of timefile          *
C* E.Nelkin/SSAI        10/11  Do not exit(0) if no unique scans found;*
C*                             let main program skip to next file if   *
C*                             istrt = istop = 0                       *
C* E.Nelkin/SSAI        10/11  Pass in cjobid                          *
C* E.Nelkin/SSAI        10/11  Added separate do_debug_selectunique    *
C* E.Nelkin/SSAI        10/11  Pass in do_debug                        *
C* E.Nelkin/SSAI        11/11  If istrt = istop, return without writing*
C*                             a new entry to the times-processed file *
C* E.Nelkin/SSAI         9/12  Non-PPS version for Bob Joyce           *
C***********************************************************************
	IMPLICIT	NONE
C
        INCLUDE         'parameters.inc'
	INTEGER*4	MXCT, MXGAP, MXDIFF
	PARAMETER	( MXCT  = 99,
     +			  MXGAP = 60,
     +			  MXDIFF = 2678400 )
C
C	Parameter values are chosen as: largest 2-digit number, 1 minute
C	(in seconds), 1 month (in seconds).
C
	INTEGER*4	kct (2, MXCT), nct, jct, iscan, iscansec, i, j, 
     +			jj, jlim, istrt, istop, nscansec, istopsec,
     +			jdrop
	INTEGER*4	nscan, l_st, iret
C       INTEGER*4       itime87(MAX_SCANS)
 	INTEGER*4       jdate(MAX_SCANS)
 	REAL*4          atime(MAX_SCANS)
	CHARACTER*256   cjobid
C
        LOGICAL         do_debug
C NOTE: do_debug_selectunique is set in parameters.inc.
C
C	CHARACTER*256   timefile
	INTEGER*4       iunit_t
C
C	Initialization in each run.
C
        REWIND(iunit_t,err=1102)
        read(iunit_t,200,err=1101) nct
  200   format(i2)
        do j = 1, nct
           read(iunit_t,201,err=1101) kct(1,j), kct(2,j)
  201      format(i9,2x,i9)
        end do
C        CALL ST_LSTR  ( timefile, l_st, iret )
C        IF (iret .NE. 0) THEN
C           IF (do_debug) WRITE (*,*)
C     +         'ERROR: UNABLE TO DETERMINE LENGTH OF SSMI ',
C     +         'PROCESSED-TIMES FILE NAME ', timefile, ' ;TERMINATING ',
C     +         'EXECUTION; SUBROUTINE SELECT_UNIQUE'
C           ierroutn = E_3B42_ERROR_IN_FILE_NAME_LENGTH
C           call reprt_error(ierroutn)
C        ENDIF
C        open(27,file=timefile(1:l_st),status='old',err=1100)
C        read(27,200,err=1101) nct
C  200   format(i2)
C        do j = 1, nct
C           read(27,201,err=1101) kct(1,j), kct(2,j)
C  201      format(i9,2x,i9)
C        end do 
	jct   = 1
	iscan = 1
	istrt = 0
	istop = 0
C
C	At start of current contact.  Seek end of overlapping listed 
C	contact.  Note: We start here, which ignores the case of the 
C	current contact coming before the oldest listed contact.
C	This avoids processing duplicate files, but can cause loss
C	of unique data if the file arrives out of order and "very late."
C
   10	DO  i = iscan, nscan
C  <convert scan time i to sec. since 1/1/1987, load into iscansec>
            CALL CONVTIME (jdate(i), atime(i), iscansec)
C           iscansec = itime87(i)
	    IF  ( iscansec .GT. kct (2, jct) )  GO TO 20
	END DO
C
C	Exhausted current contact without getting a unique scan.  Exit.
C
        if (do_debug_selectunique) then
        WRITE (*, *) 'Note: Ran through data without finding unique ',
     +               'scans at jct =', jct
        endif
C
C EJN 10/2011 - Do not exit(0) in this case; allow calling program to
C               advance to next input file if istrt = istop = 0.
C
CC      call exit(0)
        istrt = 0
        istop = 0
        REWIND(iunit_t,err=1102)
        RETURN
C
C	Found end of overlapping listed contact.  If no additional 
C	listed contacts, we're "appending" to the time span.  Set bounds 
C	on current contact and jump to wrapping up contact list.
C
   20	IF  ( jct .EQ. nct )  THEN
            if (do_debug_selectunique) then
	    WRITE (*, *) 'Note: Start appending scans at scan,jct =', 
     +			 i, jct
            endif
	    istrt = i
	    istop = nscan
	    jct   = jct + 1
            nct   = nct + 1
	    kct (1, jct) = iscansec
            CALL CONVTIME (jdate(nscan), atime(nscan), nscansec)
C           nscansec = itime87(nscan)
	    kct (2, jct) = nscansec
C
C	    Warn if the new contact is "too far" past the end of the 
C	    current list.
C	    *** Promote to PPS message in next version. ***
C
	    IF  ( kct (1, jct) .GT. kct (2, jct-1) + MXDIFF )  THEN
		IF  ( do_debug_selectunique .AND.
     +                kct(2, jct-1) .NE. 0 )  THEN
		WRITE (*, *) 'Warning: SELECT_UNIQUE detected times ',
     +			'more than', MXDIFF, ' sec past the current ',
     +			'last time processed'
                END IF
            END IF
C
C	  Do have additional listed contacts; if after start of next 
C	  listed contact, jump back to "seek end of overlapping listed
C	  contact."
C
	  ELSE IF  ( iscansec .GE. kct (1, jct+1) )  THEN
	    jct   = jct + 1
	    iscan = i
	    GO TO 10
C
C	  Start of current contact is before start of next listed 
C	  contact.  Do a "fill":  Move all subsequent listed contacts
C	  to one greater index and insert the current contact in the
C	  "next" slot.
C
	  ELSE
	    DO  j = nct, jct+1, -1
		kct (1, j+1) = kct (1, j)
		kct (2, j+1) = kct (2, j)
	    END DO
            jct          = jct + 1
	    nct          = nct + 1
	    kct (1, jct) = iscansec
	    iscan        = i
C
C	    Seek first overlap of current contact and next listed
C	    contact.
C
	    DO  i = iscan, nscan
                CALL CONVTIME (jdate(i), atime(i), iscansec)
C               iscansec = itime87(i)
		IF  ( iscansec .GE. kct (1, jct+1) )  GO TO 30
	    END DO
C
C	    Dropping through the DO means then entire remaining contact
C	    is unique.  All we need to do is ensure that i has the right
C	    value for the following code.
C
	    i = nscan + 1
C
C	    Set bounds on current contact and jump down to wrapping up
C	    contact list.
C	    
C
   30	    istrt = iscan
	    istop = i - 1
            CALL CONVTIME (jdate(istop), atime(istop), istopsec)
C           istopsec = itime87(istop)
	    kct (2, jct) = istopsec
            if (do_debug_selectunique) then
	    WRITE (*, *) 'Note: Filling with start,stop,jct =', 
     +			 istrt, istop, jct
            endif
	END IF
C
C	The IF's are exhaustive; "fill" catches everything that
C	isn't "seeking end of overlap" or "append."  Now wrap up the 
C	contact list.  First, if nct=MXCT the list is max'ed out; remove
C	the oldest contact by shifting all the rest down and decrement 
C	nct to guarantee room for the next run.  This is how we age off 
C	gaps that are "really old" and (might) never be filled.  This 
C	action comes first because the rest of the list might be 
C	contiguous and can be pared down in the following code.
C
	IF  ( nct .GE. MXCT )  THEN
            if (do_debug_selectunique) then
	    WRITE (*, *) 'Note: Contact list full; pitch oldest item.'
            endif
	    DO  j = 1, MXCT-1
		kct (1, j) = kct (1, j+1)
		kct (2, j) = kct (2, j+1)
	    END DO
	    nct = MXCT - 1
	END IF
C
C	For each entry except the last, if the end time is less than 
C	MXGAP seconds before the next start time, move all entries after
C	the first down by one index value and decrement nct.  The 1/07
C	"jdrop" fix moves the comparison down by one as well.  The upper
C	limit on the DO is a temporary variable to avoid confusion
C	with decrementing nct inside the loop.
C
	jlim  = nct - 1
	jdrop = 0
	DO  j = 1, jlim
	    IF  ( kct (1, j+1-jdrop) - kct (2, j-jdrop) .LT. MXGAP )  
     +								THEN
                if (do_debug_selectunique) then
		WRITE (*, *) 'Note: Pitching contiguous contact at j =',
     +			     j
                endif
		DO  jj = 1, nct-1
		    kct (1, jj) = kct (1, jj+1)
		    kct (2, jj) = kct (2, jj+1)
		END DO
		nct   = nct   - 1
		jdrop = jdrop + 1
C
C	      Else if the end time is at least MXGAP seconds before the 
C	      next start time, jump to writing out the contact time 
C	      file.
C
	      ELSE
                if (do_debug_selectunique) then
	        WRITE (*, *) 'Note: Leaving contact due to gap at j =',
     +			     j
                endif
		GO TO 40
	    END IF
	END DO
C
C	Write out the contact time file and pass istrt, istop back to
C       ingest_sdr.
C
   40  	REWIND(iunit_t,err=1102)
C
C       If istrt = istop (even if not zero), meaning only one unique
C       scan was found, return without writing an entry to the times-
C       processed file.  The calling program then returns to the driver
C       program to get the next input file if istrt = istop.
C
        IF ( istrt .EQ. istop ) RETURN
C
        write(iunit_t,200,err=1103) nct
        do j = 1, nct
           write(iunit_t,201,err=1103) kct(1,j), kct(2,j)
        end do
C
        iret = 0
        RETURN
C
C       close(27)
C
C  Report opening/reading/rewinding/writing data file errors.
C
C1100   IF (do_debug) WRITE(*,*)
C    &                'Error opening NRT times processed file!'
C       ierroutn = E_3B42_ERROR_OPENING_NRT_TIMES_FILE
C       call reprt_error(ierroutn)
 1101   IF (do_debug) WRITE(*,*)
     &     'ERROR: PROBLEM READING PROCESSED-TIMES FILE ',
     &     '; TERMINATING EXECUTION; SUBROUTINE SELECT_UNIQUE'
        STOP
 1102   IF (do_debug) WRITE(*,*)
     &     'ERROR: PROBLEM REWINDING PROCESSED-TIMES FILE ',
     &     '; TERMINATING EXECUTION; SUBROUTINE SELECT_UNIQUE'
        STOP
 1103   IF (do_debug) WRITE(*,*)
     &     'ERROR: PROBLEM WRITING TO PROCESSED-TIMES FILE ',
     &     '; TERMINATING EXECUTION; SUBROUTINE SELECT_UNIQUE'
        STOP
        RETURN
	END
